package net.hostsharing.cas.auth;

import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.net.URL;
import java.security.GeneralSecurityException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.xmlrpc.XmlRpcException;
import org.apache.xmlrpc.client.XmlRpcClient;
import org.apache.xmlrpc.client.XmlRpcClientConfigImpl;
import org.apereo.cas.authentication.AuthenticationHandlerExecutionResult;
import org.apereo.cas.authentication.CoreAuthenticationUtils;
import org.apereo.cas.authentication.PreventedException;
import org.apereo.cas.authentication.credential.UsernamePasswordCredential;
import org.apereo.cas.authentication.handler.support.AbstractUsernamePasswordAuthenticationHandler;
import org.apereo.cas.authentication.principal.Principal;
import org.apereo.cas.authentication.principal.PrincipalFactory;
import org.apereo.cas.services.ServicesManager;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class HostsharingAuthenticationHandler extends AbstractUsernamePasswordAuthenticationHandler {

	public HostsharingAuthenticationHandler(String name, ServicesManager servicesManager, PrincipalFactory principalFactory) {
		super(name, servicesManager, principalFactory, Integer.MAX_VALUE);
	}

	@Override
	protected AuthenticationHandlerExecutionResult authenticateUsernamePasswordInternal(UsernamePasswordCredential credential, String originalPassword)
			throws GeneralSecurityException, PreventedException {

		final String username = credential.getUsername();
		final String password = credential.getPassword();

		try {

			final Map<String, List<Object>> attributes = validateCredentials(username, password);
			final Principal principal = this.principalFactory.createPrincipal(username, attributes);
			return createHandlerResult(credential, principal);
			
		} catch (PasswordValidationException | IOException | XmlRpcException | ParserConfigurationException | SAXException e) {
			throw new GeneralSecurityException(e);
		}		
	}

	private static Map<String, List<Object>> validateCredentials(final String login, final String password)
			throws PasswordValidationException, XmlRpcException, GeneralSecurityException, IOException, ParserConfigurationException, SAXException {
		
		if (!login.contains("@")) {
			throw new GeneralSecurityException("expect email address");
		}
		final String emailDomain = login.split("@")[1];
		final URL url = new URL("http://" + emailDomain + "/.well-known/autoconfig/mail/config-v1.1.xml?emailaddress=" + login);
		final InputStream autoconfigStream = url.openConnection().getInputStream();		
		final DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
		final DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
		final Document document = documentBuilder.parse(autoconfigStream);
		final NodeList inServersNodes = document.getElementsByTagName("username");
		if (inServersNodes.getLength() != 2) {
			throw new GeneralSecurityException("expect email address");
		}
		
		final String username = inServersNodes.item(0).getTextContent();
		
		
		final TicketService ticketService = new TicketService(username, password);
		final String grantingTicket = ticketService.getGrantingTicket();
		final String ticket = ticketService.getServiceTicket(grantingTicket);
		
		final XmlRpcClientConfigImpl config = new XmlRpcClientConfigImpl();
		config.setServerURL(new URL("https://config.hostsharing.net:443/hsar/xmlrpc/hsadmin"));
		config.setEnabledForExtensions(true);
		final XmlRpcClient client = new XmlRpcClient();
		client.setConfig(config);

		final List<Serializable> xmlRpcParamsList = new ArrayList<Serializable>();
		xmlRpcParamsList.add(username);
		xmlRpcParamsList.add(ticket);
		final HashMap<String, Serializable> whereParamsMap = new HashMap<String, Serializable>();
		whereParamsMap.put("name", username);
		xmlRpcParamsList.add(whereParamsMap);
		final Object[] rpcResult = (Object[]) client.execute("user.search", xmlRpcParamsList);
		if (rpcResult.length != 1) {
			throw new GeneralSecurityException("unknown username");
		}
		@SuppressWarnings("unchecked")
		final Map<String, Serializable> userData = (Map<String, Serializable>) rpcResult[0];
		final String comment = (String) userData.get("comment");
		int firstCommaIndex = comment.indexOf(',');
		String displayName = comment;
		String[] groups = new String[0];
		if (firstCommaIndex > 0) {
			displayName = comment.substring(0, firstCommaIndex).trim();
			final String[] splitStrings = comment.substring(firstCommaIndex + 1).split(",");
			groups = new String[splitStrings.length];
			for (int idx=0; idx<splitStrings.length; idx++) {
				groups[idx] = splitStrings[idx].trim();
			}
		}
		final Map<String, Object> attribsMap = new HashMap<String, Object>();
		attribsMap.put("groups", groups);
		attribsMap.put("displayName", displayName);
		attribsMap.put("mail", login);
		final Map<String, List<Object>> attributes = CoreAuthenticationUtils.convertAttributeValuesToMultiValuedObjects(attribsMap);
		return attributes;
	}

	public static void main(String[] args) {
		try {
			Map<String, List<Object>> map = validateCredentials(args[0], args[1]);
			for (String key : map.keySet()) {
				System.out.println(key + ": " + map.get(key));
			}
		} catch (IOException | PasswordValidationException | XmlRpcException | GeneralSecurityException | ParserConfigurationException | SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
